package org.beangle.webapp.portal.action;

import static org.apache.commons.lang.StringUtils.trim;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.struts2.interceptor.ServletRequestAware;
import org.beangle.security.monitor.AuthenticationException;
import org.beangle.security.monitor.SecurityManager;
import org.beangle.security.monitor.auth.provider.UserNamePasswordAuthentication;
import org.beangle.security.monitor.auth.web.WebUserDetailsSource;
import org.beangle.struts2.action.BaseAction;

import com.octo.captcha.service.CaptchaService;
import com.octo.captcha.service.CaptchaServiceException;

public class LoginAction extends BaseAction implements ServletRequestAware {

	private SecurityManager securityManager;

	private HttpServletRequest request;

	private LoginForm loginForm;

	private CaptchaService captchaService;

	public static final String LOGIN_FAILURE_COUNT = "loginFailureCount";

	public String index() {
		if (!shouldLogin()) {
			return "failure";
		}
		String errorMsg = doLogin(loginForm);
		if (StringUtils.isNotEmpty(errorMsg)) {
			addActionError(getText(errorMsg));
			increaseLoginFailure();
			return "failure";
		}
		clearLoginFailure();
		return "home";
	}

	protected boolean shouldLogin() {
		if (null == loginForm) {
			return false;
		}
		if (notFailEnough()) {
			return true;
		}
		// 校验验证码
		if (null != captchaService) {
			try {
				String sessionId = request.getSession().getId();
				Boolean valid = captchaService.validateResponseForID(sessionId, loginForm
						.getCaptcha());
				if (Boolean.FALSE.equals(valid)) {
					addActionError(getText("error.captcha"));
					return false;
				}
			} catch (CaptchaServiceException e) {
				addActionError(getText("error.captcha"));
				return false;
			}
		}
		return true;
	}

	protected String doLogin(LoginForm loginForm) {
		loginForm.setName(trim(loginForm.getName()));
		UserNamePasswordAuthentication auth = new UserNamePasswordAuthentication(loginForm
				.getName(), loginForm.getPassword());
		auth.setRequest(request);
		auth.setDetails(new WebUserDetailsSource().buildDetails(request));
		try {
			securityManager.authenticate(auth);
		} catch (AuthenticationException e) {
			return e.getMessage();
		}
		return "";
	}

	private boolean notFailEnough() {
		Integer loginFailureCount = (Integer) request.getSession()
				.getAttribute(LOGIN_FAILURE_COUNT);
		if (null == loginFailureCount) {
			loginFailureCount = new Integer(0);
		}
		if (loginFailureCount.intValue() <= 1) {
			return true;
		}
		return false;
	}

	private void increaseLoginFailure() {
		Integer loginFailureCount = (Integer) request.getSession()
				.getAttribute(LOGIN_FAILURE_COUNT);
		if (null == loginFailureCount) {
			loginFailureCount = new Integer(0);
		}
		loginFailureCount++;
		request.getSession().setAttribute(LOGIN_FAILURE_COUNT, loginFailureCount);
	}

	private void clearLoginFailure() {
		request.getSession().removeAttribute(LOGIN_FAILURE_COUNT);
	}

	public void setSecurityManager(SecurityManager securityManager) {
		this.securityManager = securityManager;
	}

	public LoginForm getLoginForm() {
		return loginForm;
	}

	public void setLoginForm(LoginForm loginForm) {
		this.loginForm = loginForm;
	}

	public void setServletRequest(HttpServletRequest request) {
		this.request = request;
	}

	public void setCaptchaService(CaptchaService captchaService) {
		this.captchaService = captchaService;
	}

}
